#!/usr/bin/env python
"""
Once submit has finished with the jobs, this function is called to have PUQ
process the results.
"""
import sys
import os
import numpy as np
import h5py
import re
from puq.jpickle import unpickle
import xml.etree.ElementTree as xml

# Redirect stdout and stderr to files for debugging.
# Append to the files created in get_params.py
sys.stdout = open("uq_debug.out", 'a')
sys.stderr = open("uq_debug.err", 'a')

# Restore the state of a PUQ session from a HDF5 file.
def load_from_hdf5(name):
    h5 = h5py.File(name, 'r+')
    sw = unpickle(h5['private/sweep'].value)
    sw.fname = os.path.splitext(name)[0]
    h5.close()

    sw.psweep._sweep = sw

    if hasattr(sw.psweep, 'reinit'):
        sw.psweep.reinit()
    return sw


# Plots probability curves
def plot_pdf_curve(xvals, vname, percent, desc):
    print 'plot_pdf_curve %s %s %s' % (vname, percent, desc)
    # compute upper and lower percentiles
    pm = (100 - percent)/200.0
    pp = 1 - pm

    # collect data into an array
    xarr = np.empty(len(xvals[vname]))
    yp = np.empty(len(xvals[vname]))
    ym = np.empty(len(xvals[vname]))
    for vindex in sorted(xvals[vname].keys()):
        xarr[vindex] = xvals[vname][vindex]
        yp[vindex] = pcurves[vname][vindex].ppf(pp)
        ym[vindex] = pcurves[vname][vindex].ppf(pm)

    curve = xml.SubElement(dout, 'curve', {'id': 'curve_pdf-%s-%s' % (vname, percent)})
    about = xml.SubElement(curve, 'about')
    if percent == 0:
        xml.SubElement(about, 'label').text = "mean"
    else:
        xml.SubElement(about, 'label').text = "middle %s%%" % percent
    xml.SubElement(about, 'group').text = '%s (Probability)' % desc
    comp = xml.SubElement(curve, 'component')
    xy = xml.SubElement(comp, 'xy')
    pts = ""
    for x, y in zip(xarr, yp):
        pts += "%s %s " % (x, y)
    if percent == 0:
        pts += '\n'
    else:
        for x, y in reversed(zip(xarr, ym)):
            pts += "%s %s " % (x, y)
        pts += "%s %s\n" % (xarr[0], yp[0])
    xy.text = pts


def dist(x1,y1, x2,y2, x3,y3): # x3,y3 is the point
    diffx = x2-x1
    diffy = y2-y1
    sqdiff = float(diffx**2 + diffy**2)

    u = ((x3 - x1) * diffx + (y3 - y1) * diffy) / sqdiff

    if u > 1:
        u = 1
    elif u < 0:
        u = 0

    x = x1 + u * diffx
    y = y1 + u * diffy

    dx = x - x3
    dy = y - y3

    return np.sqrt(dx*dx + dy*dy)

# Plots advanced probability curves
def plot_pdf_acurve(hf, acurves, vname, percent, desc):
    print 'plot_pdf_acurve %s %s %s' % (vname, percent, desc)
    # compute upper and lower percentiles
    pm = (100 - percent)/200.0
    pp = 1 - pm
    vlen = len(acurves[vname])

    print 'vlen=', vlen
    # collect data into an array
    xp = np.empty(vlen)
    xm = np.empty(vlen)
    yp = np.empty(vlen)
    ym = np.empty(vlen)

    for vindex in sorted(acurves[vname].keys()):
        print 'vindex=', vindex
        x1 = acurves[vname][vindex][0].ppf(pp)
        x2 = acurves[vname][vindex][0].ppf(pm)
        y1 = acurves[vname][vindex][1].ppf(pp)
        y2 = acurves[vname][vindex][1].ppf(pm)
        xd = hf['/output/data/%s.x[%d]' % (vname, int(vindex))].value
        yd = hf['/output/data/%s.y[%d]' % (vname, int(vindex))].value
        #p1, p2 = get_closest2()
        if int(vindex) == 2:
            print x1, x2, y1, y2
            print 'xd=', repr(xd)
            print 'yd=', repr(yd)

        #xp[vindex] = x1
        #xm[vindex] = x2
        #yp[vindex] = y1
        #ym[vindex] = y2
    return
    curve = xml.SubElement(dout, 'curve', {'id': 'curve_pdf-%s-%s' % (vname, percent)})
    about = xml.SubElement(curve, 'about')
    if percent == 0:
        xml.SubElement(about, 'label').text = "mean"
    else:
        xml.SubElement(about, 'label').text = "middle %s%%" % percent
    xml.SubElement(about, 'group').text = '%s (Probability)' % desc
    comp = xml.SubElement(curve, 'component')
    xy = xml.SubElement(comp, 'xy')
    pts = ""
    for x, y in zip(xarr, yp):
        pts += "%s %s " % (x, y)
    if percent == 0:
        pts += '\n'
    else:
        for x, y in reversed(zip(xarr, ym)):
            pts += "%s %s " % (x, y)
        pts += "%s %s\n" % (xarr[0], yp[0])
    xy.text = pts


def plot_pdf(v, pdf, desc):
    id = '%s (PDF)' % desc
    elem = xml.SubElement(dout, 'curve', {'id': id})
    about = xml.SubElement(elem, 'about')
    xml.SubElement(about, 'label').text = id
    yaxis = xml.SubElement(elem, 'yaxis')
    xml.SubElement(yaxis, 'label').text = 'Probability'

    component = xml.SubElement(elem, 'component')
    xy = xml.SubElement(component, 'xy')
    pts = "%s 0\n" % pdf.x[0]
    for x, y in zip(pdf.x, pdf.y):
        pts += "%s %s\n" % (x, y)
    pts += "%s 0\n" % pdf.x[-1]
    xy.text = pts


sw = load_from_hdf5(sys.argv[1])
sw.analyze()

h5 = h5py.File(sys.argv[1], 'r+')
dtree = xml.parse('run_uq.xml')
droot = dtree.getroot()
dout = droot.find('output')

# curves built from pdfs
pcurves = {}
xvals = {}
acurves = {}

reg1 = re.compile('([ \da-zA-Z]+)\[([ \d]+)\]')
reg2 = re.compile('([ \da-zA-Z]+)\.([xy])\[([ \d]+)\]')

uqtype = h5.attrs['UQtype']
for v in h5[uqtype]:
    rs = unpickle(h5['/%s/%s/response' % (uqtype, v)].value)
    pdf = rs.pdf(fit=False)
    odata = h5['/output/data/%s' % v]

    # For curves built from pdfs, just put them in a dict for now
    if 'x' in odata.attrs:
        matches = reg1.findall(v)
        vname, vindex = matches[0]
        vindex = int(vindex)
        if vname not in pcurves:
            pcurves[vname] = {}
            xvals[vname] = {}
        xvals[vname][vindex] = odata.attrs['x']
        pcurves[vname][vindex] = pdf
    elif reg2.findall(v) != []:
        matches = reg2.findall(v)
        vname, xy, vindex = matches[0]
        if vname not in acurves:
            acurves[vname] = {}
        if vindex not in acurves[vname]:
            acurves[vname][vindex] = {}
        if xy == 'x':
            acurves[vname][vindex][0] = pdf
        else:
            acurves[vname][vindex][1] = pdf
    else:
        desc = h5['/output/data/%s' % v].attrs['description']
        plot_pdf(v, pdf, desc)

# now do pdf curves
for vname in xvals:
    desc = h5['/output/data/%s[0]' % vname].attrs['description']
    plot_pdf_curve(xvals, vname, 0, desc)
    plot_pdf_curve(xvals, vname, 50, desc)
    plot_pdf_curve(xvals, vname, 95, desc)

#for vname in acurves:
#    desc = h5['/output/data/%s.x[0]' % vname].attrs['description']
#    plot_pdf_acurve(h5, acurves, vname, 0, desc)
#    plot_pdf_acurve(h5, acurves, vname, 50, desc)
#    plot_pdf_acurve(h5, acurves, vname, 95, desc)

with open('run_uq.xml', 'w') as f:
    f.write("<?xml version=\"1.0\"?>\n")
    dtree.write(f)

with open('run_uq2.xml', 'w') as f:
    f.write("<?xml version=\"1.0\"?>\n")
    dtree.write(f)
h5.close()
